/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.auth;

import org.apache.commons.lang3.StringUtils;
import org.opengroup.osdu.core.common.model.entitlements.AuthorizationResponse;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.provider.interfaces.IAuthorizationService;
import org.opengroup.osdu.notification.di.RequestInfoExt;
import org.opengroup.osdu.notification.utils.Config;
import org.opengroup.osdu.notification.provider.interfaces.IServiceAccountValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

@Component("authorizationFilter")
@RequestScope
public class AuthorizationFilter {
    private static final String BEARER_PREFIX = "Bearer ";

    @Autowired
    private HttpServletRequest request;
    @Autowired
    private IAuthorizationService authService;
    @Autowired
    private RequestInfoExt requestInfoExt;
    @Autowired
    private IServiceAccountValidator validator;

    public boolean hasAnyPermission(String... requiredRoles) {
        DpsHeaders dpsHeaders = requestInfoExt.getHeaders();
        if (StringUtils.isBlank(dpsHeaders.getAuthorization()))
            throw new AppException(302, "Redirect", "No credentials sent on request.");

        if (StringUtils.isBlank(dpsHeaders.getPartitionId())) {
            requestInfoExt.assignPartitionIdIfNotInHeader();
        }

        String path = request.getServletPath();
        if ("GET".equals(request.getMethod()) &&
           (path.equals("/swagger-ui.html") || path.equals("/info"))) {
            return true;
        }

        if (Arrays.asList(requiredRoles).contains(Config.CRON) && requestInfoExt.isCronRequest()) {
            dpsHeaders.put(DpsHeaders.USER_EMAIL, Config.CRON);
            requestInfoExt.setHeaders(dpsHeaders);
        } else if (Arrays.asList(requiredRoles).contains(Config.PUBSUB)) {
            String jwt = dpsHeaders.getAuthorization().substring(BEARER_PREFIX.length());
            if (!this.validator.isValidPublisherServiceAccount(jwt)) {
                this.authorizeWithEntitlements(requiredRoles);
            } else {
                return false;
            }
        } else {
            authorizeWithEntitlements(requiredRoles);
        }
        return true;
    }

    private void authorizeWithEntitlements(String... requiredRoles) {
        DpsHeaders dpsHeaders = requestInfoExt.getHeaders();
        AuthorizationResponse authorizationResponse = authService.authorizeAny(dpsHeaders, requiredRoles);
        dpsHeaders.put(DpsHeaders.USER_EMAIL, authorizationResponse.getUser());
        requestInfoExt.setHeaders(dpsHeaders);
    }
}
