/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.pubsub.di;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Strings;
import com.google.gson.Gson;
import org.apache.http.HttpStatus;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.notification.ISubscriptionFactory;
import org.opengroup.osdu.core.common.notification.ISubscriptionService;
import org.opengroup.osdu.core.common.notification.SubscriptionException;
import org.opengroup.osdu.core.common.util.IServiceAccountJwtClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
@ConditionalOnProperty(name = "oqmDriver")
public class OqmSubscriptionHandler {
    @Autowired
    private ISubscriptionFactory subscriptionFactory;
    @Autowired
    private OqmSubscriptionCacheFactory subscriptionCacheFactory;
    @Autowired
    private JaxRsDpsLog log;

    @Autowired
    private IServiceAccountJwtClient serviceAccountJwtClient;

    private static final Gson gson = new Gson();
    private ObjectMapper objectMapper;

    public Subscription getSubscriptionFromCache(String notificationId, Map<String, String> headerAttributes) {
        String subscriptionString = subscriptionCacheFactory.get(notificationId, headerAttributes);
        try {
            if (Strings.isNullOrEmpty(subscriptionString))
                subscriptionString = querySubscriptionAndUpdateCache(notificationId, headerAttributes);
            ObjectMapper objectMapper = this.getObjectMapper();
            return objectMapper.readValue(subscriptionString, Subscription.class);
        } catch (IOException e) {
            throw new AppException(HttpStatus.SC_INTERNAL_SERVER_ERROR, "Error Parsing subscription String to object", "Unexpected error in pushing message", e);
        } catch (SubscriptionException se) {
            throw new AppException(HttpStatus.SC_INTERNAL_SERVER_ERROR, "Error query subscription from registration", "Unexpected error in pushing message", se);
        }
    }

    private String querySubscriptionAndUpdateCache(String notificationId, Map<String, String> headerAttributes) throws AppException, SubscriptionException {
        DpsHeaders headers = getDpsHeaders(headerAttributes);
        ISubscriptionService service = subscriptionFactory.create(headers);

        List<Subscription> subscriptionList = service.query(notificationId);
        if (subscriptionList == null || subscriptionList.size() == 0) {
            throw new AppException(HttpStatus.SC_NOT_FOUND, "Not found subscription for notificationId:" + notificationId, "Subscription not found");
        }

        Subscription subscription = subscriptionList.get(0);
        String jsonSubscription = gson.toJson(subscription);
        this.subscriptionCacheFactory.put(subscription.getNotificationId(), jsonSubscription, headerAttributes);

        return jsonSubscription;
    }

    private DpsHeaders getDpsHeaders(Map<String, String> headerAttributes) {
        Map<String, String> attributes = new HashMap<>(headerAttributes);

        //extract headers from pubsub message
        String dataPartitionId = headerAttributes.get(DpsHeaders.DATA_PARTITION_ID);
        String authToken = this.serviceAccountJwtClient.getIdToken(dataPartitionId);
        attributes.put(DpsHeaders.AUTHORIZATION, authToken);
        return DpsHeaders.createFromMap(attributes);

    }

    //unit test purpose
    protected ObjectMapper getObjectMapper() {
        if (this.objectMapper == null) {
            this.objectMapper = new ObjectMapper();
        }
        return this.objectMapper;
    }

    //unit test purpose
    void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }
}
